

/***************************************************
  RF tester Ver. 2.01
    RF Analyzer
    Arduino-nano IDE 1.8.1
    LogAmp AD8307
    DDS AD9851
              2017.12.24 JA2NKD Ryuu Matsuura
    Copyright(C)2017.JA2NKD.All rights reserved.  
 ****************************************************/


#include "SPI.h"
#include "Ucglib.h"       // Ver 1.4.0
#include <Rotary.h>      // Ver ???
#include <EEPROM.h>

/*
 * Hardware SPI Pins:
 * Arduino nano sclk=13, data=11 
*/

  const byte __CS=10;
  const byte __DC=9;
  const byte __RST=8;
  const byte DDS_RST = 4;
  const byte DDS_DATA = 5;
  const byte DDS_FQ_UD = 6;
  const byte DDS_W_CLK = 7;
  const unsigned long TWO_E=4294967295L;     // 2e32 AD9850,9851
  const byte DDS_CMD = B00000001;   // AD9851 B00000001
  const byte AD8307 = A5;
  const byte sw_menu = A0;
  const byte sw_enter = A1;
  const byte sw_right = A3;
  const byte sw_left = A2;
  Ucglib_ILI9341_18x240x320_HWSPI ucg(__DC, __CS, __RST);
  Rotary r=Rotary(2,3);  
  unsigned long DDS_CLK = 180000000L;   // AD9851 180000000
  long DDS_freq =0;
  long DDSfmin =0;
  long DDSfminold =0;
  long DDSfmax =70000000;
  long DDSfmaxold =70000000;
  float DDSfpit =0;
  int fcountflg=0;
  int ADOUT=0;
  int ADOUTmax=0;
  int ADOUTmin=1023;
  long ADOUTmaxf=0;
  long ADOUTminf=0;
  int xold=10;
  int yold=230;
  float dBm=0;
  float dBmold=100;
  int menun=1;
  long fstep=10000000;
  int fstepflg=8;
  long SGfreq=10000000;
  long SGfreqold=70000000;
  int configflg=0;
  int mode=0;
  long offsetpix=0;
  long offsetpixold =0;
  int romcheck=0;
  float AD_a=0.09;
  float AD_b=-86.06;
  float swr=1;
  float rloss=1;
  int ADmaxi=0;
  int ADmini=0;

void setup() {

  pinMode(DDS_RST,OUTPUT);      // DDS pin set OUTUT
  pinMode(DDS_DATA,OUTPUT);
  pinMode(DDS_FQ_UD,OUTPUT);
  pinMode(DDS_W_CLK,OUTPUT);
  pinMode(sw_menu,INPUT_PULLUP);
  pinMode(sw_enter,INPUT_PULLUP);  
  pinMode(sw_right,INPUT_PULLUP);
  pinMode(sw_left,INPUT_PULLUP);
  analogReference(EXTERNAL);

  if ( digitalRead(sw_enter) == LOW){
    eepinitwrite();
  }     
  romcheck=EEPROM.read(0x001);
  if (romcheck !=123 ){
    eepinitwrite();   
    EEPROM.write(0x001,123);
  }

  else{
    EEPROM.get(0x010,DDS_CLK);
    EEPROM.get(0x100,AD_a);
    EEPROM.get(0x120,AD_b);
    EEPROM.get(0x070,SGfreq);
    
  }
  
  ucg.begin(UCG_FONT_MODE_TRANSPARENT);
//  ucg.setRotate270();
  ucg.setRotate90();

  
  PCICR |=(1<<PCIE2);
  PCMSK2 |=(1 << PCINT18) | (1 << PCINT19);
  sei();
  DDS_init();
  menu1();
  menu2(); 
}

void loop(void) {
  if (digitalRead(sw_menu)==LOW){
    menun=menun+1;
    if (menun>=8){menun=1;}
    menu2();
    while(digitalRead(sw_menu) == LOW){}
  }
  if (digitalRead(sw_enter) == LOW){
    switch (menun){
      case 1:
      signalg();
        break;
      case 2:
        ucg.clearScreen();
        powermeter();
        break;
      case 3:
        ucg.clearScreen();
         mode=3;
         FRM();
        break;
      case 4:
        ucg.clearScreen();
        mode=4;
        FRM();
        break;
      case 5:
        ucg.clearScreen();      
        DDSajust();
        break;
      case 6:
        ucg.clearScreen();
        ADcheck();
        break;
    }
  }
}

// ------------ Encoder Interrupt ------------------------
  ISR(PCINT2_vect) {
  float minfprint=0;
  float maxfprint=0;    
  unsigned char result = r.process();
      if(result) {
        if(result == DIR_CW){
          if (mode==1){
            SGfreq=SGfreq+fstep;
          }
          if ((mode==3) or(mode==4)){
            ucg.setColor(0,0,0);
            ucg.drawBox(165,172,140,30);
            ucg.setColor(0,240,240);
            if (configflg==1){
              DDSfmin=DDSfmin+fstep;
              DDSfmin=constrain(DDSfmin,0,70000000);
              ucg.setPrintPos(180,195);
              minfprint=(float)DDSfmin/1000;
              ucg.print(minfprint,3);
            }   
            if (configflg==2){
              DDSfmax=DDSfmax+fstep;
              DDSfmax=constrain(DDSfmax,0,70000000);
              ucg.setPrintPos(180,195);
              maxfprint=(float)DDSfmax/1000;
              ucg.print(maxfprint,3);
            }
            if (configflg==3){
              offsetpix=offsetpix+fstep;
              offsetpix=constrain(offsetpix,-30,30);
              ucg.setPrintPos(180,195);
              ucg.print(offsetpix);
            }
          }
          if(mode == 5){
            DDS_CLK=DDS_CLK+fstep;
            DDS_CLK=constrain(DDS_CLK,179000000,181000000);
            ucg.setColor(0,0,0);
            ucg.drawBox(165,172,140,30);
            ucg.setColor(0,240,240);
            ucg.setPrintPos(180,195);
            ucg.print(DDS_CLK);
            DDS_freqset(10000000);
            EEPROM.put(0x10,DDS_CLK);    
          } 
      }

      else{
        if (mode==1){
          SGfreq=SGfreq-fstep;
        }
        if ((mode==3) or(mode==4)){
           ucg.setColor(0,0,0);
           ucg.drawBox(165,172,140,30);
           ucg.setColor(0,240,240);
           if (configflg==1){
             DDSfmin=DDSfmin-fstep;
             DDSfmin=constrain(DDSfmin,0,70000000);
             ucg.setPrintPos(180,195);
             minfprint=(float)DDSfmin/1000;
             ucg.print(minfprint,3);
           }   
           if (configflg==2){
             DDSfmax=DDSfmax-fstep;
             DDSfmax=constrain(DDSfmax,0,70000000);
             ucg.setPrintPos(180,195);
             maxfprint=(float)DDSfmax/1000;
             ucg.print(maxfprint,3);
           }
           if (configflg==3){
            offsetpix=offsetpix-fstep;
            offsetpix=constrain(offsetpix,-30,30);
             ucg.setPrintPos(180,195);
             ucg.print(offsetpix);
            }
           }
          if (mode==5){
            DDS_CLK=DDS_CLK-fstep;
            DDS_CLK=constrain(DDS_CLK,179000000,181000000);
            ucg.setColor(0,0,0);
            ucg.drawBox(165,172,140,30);
            ucg.setColor(0,240,240);
            ucg.setPrintPos(180,195);
            ucg.print(DDS_CLK);
            DDS_freqset(10000000);
            EEPROM.put(0x010,DDS_CLK);     
          }
      }
    }
    if (mode==1){
      SGfreq = constrain(SGfreq,0,70000000);
    }
}
  
// ----------------- Main Menu -------------------------------
void menu1(){
  ucg.clearScreen();
  DDS_freqset(0);  
  mode=0;
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setPrintPos(125,35);
  ucg.print("[ MENU ]");
  ucg.setColor(255,255,255);
  ucg.setPrintPos(10,70);
  ucg.print("1. Signal Generator"); 
  ucg.setPrintPos(10,95);
  ucg.print("2. Power Meter"); 
  ucg.setPrintPos(10,120);
  ucg.print("3. Frequency Responce"); 
  ucg.setPrintPos(10,145);
  ucg.print("4. Antenna Analyzer");
  ucg.setPrintPos(10,170);
  ucg.print("5. DDS Adjustment");
  ucg.setPrintPos(10,195);
  ucg.print("6. AD8307 Adjustment");
  ucg.setPrintPos(30,235);
  ucg.setFont(ucg_font_7x14B_tr);
  ucg.setColor(0,240,240);
  ucg.print("Homebrew RF Analyzer Ver 2.01  by JA2NKD");

  
}

// ------------------- Main Menu sub ---------------------------
void menu2(){
    ucg.setFont(ucg_font_fub17_tr);
    switch (menun){
      case 1:
        ucg.setColor(255,255,255); 
        ucg.setPrintPos(10,195);
        ucg.print("6. AD8307 Adjustment");
        ucg.setColor(255,0,0);
        ucg.setPrintPos(10,70);
        ucg.print("1. Signal Generator"); 
        break;      
      case 2:
        ucg.setColor(255,255,255);
        ucg.setPrintPos(10,70);
        ucg.print("1. Signal Generator"); 
        ucg.setColor(255,0,0);
        ucg.setPrintPos(10,95);
        ucg.print("2. Power Meter"); 
        break;
      case 3:
        ucg.setColor(255,255,255);
        ucg.setPrintPos(10,95);
        ucg.print("2. Power Meter");       
        ucg.setColor(255,0,0);
        ucg.setPrintPos(10,120);
        ucg.print("3. Frequency Responce");
        break;
      case 4:
        ucg.setColor(255,255,255);
        ucg.setPrintPos(10,120);
        ucg.print("3. Frequency Responce");     
        ucg.setColor(255,0,0); 
        ucg.setPrintPos(10,145);
        ucg.print("4. Antenna Analyzer"); 
        break;
      case 5:
        ucg.setColor(255,255,255); 
        ucg.setPrintPos(10,145);
        ucg.print("4. Antenna Analyzer"); 
        ucg.setColor(255,0,0);
        ucg.setPrintPos(10,170);
        ucg.print("5. DDS Adjustment");
        break;
      case 6:
        ucg.setColor(255,255,255); 
        ucg.setPrintPos(10,170);
        ucg.print("5. DDS Adjustment"); 
        ucg.setColor(255,0,0);
        ucg.setPrintPos(10,195);
        ucg.print("6. AD8307 Adjustment");
        break;
      
     }  
}

// ---------------- Signal Generator ------------------------
void signalg(){
  SGfreqold=71000000;
  long counter=0;
  dBmold=100;
  mode=1;
  EEPROM.get(0x070,SGfreq);
  fstepflg=EEPROM.read(0x080);
//  fstepflg=7;
  ucg.clearScreen();
  ucg.setColor(255,255,255);
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setPrintPos(10,30);
  ucg.print("Signal Generator");
  ucg.setPrintPos(10,155);
  ucg.print("Power meter");
  DDS_freqset(SGfreq);
  step();
  while(digitalRead(sw_menu) ==HIGH){
    if (SGfreq != SGfreqold){
      DDS_freqset(SGfreq);
      ucg.setColor(0,0,0);
      ucg.drawBox(47,60,232,40);
      SGfreqold=SGfreq;
      ucg.setFont(ucg_font_fub35_tn);
      ucg.setColor(255,100,100);
      int xpos=50;
      if (SGfreq>=0){xpos=243;}
      if (SGfreq>=10){xpos=215;}
      if (SGfreq>=100){xpos=187;}
      if (SGfreq>=1000){xpos=159;}
      if (SGfreq>=10000){xpos=131;}
      if (SGfreq>=100000){xpos=103;}
      if (SGfreq>=1000000){xpos=75;}                 
      if (SGfreq>=10000000){xpos=47;}
      ucg.setPrintPos(xpos,100);
      ucg.print(SGfreq);
      ucg.setPrintPos(280,100);
      ucg.setColor(255,255,255);
      ucg.setFont(ucg_font_fub17_tr);  
      ucg.print("Hz");
      ucg.setFont(ucg_font_7x14B_tr);
      ucg.setColor(255,255,255) ;
      ucg.setPrintPos(95,115);     
      ucg.print("M           K");      
      }
    if (digitalRead(sw_left) == LOW){
      fstepflg=fstepflg-1;
      if (fstepflg<1){fstepflg=1;}
      step();
      EEPROM.put(0x080,fstepflg);
      while(digitalRead(sw_left)==LOW){delay(10);}       
    }
    if (digitalRead(sw_right) == LOW){
      fstepflg=fstepflg+1;
      if (fstepflg>8){fstepflg=8;}
    step();
    EEPROM.put(0x080,fstepflg);
    while(digitalRead(sw_right)==LOW){delay(10);}
    }
   if(++counter ==100000){
   ADOUT=analogRead(AD8307);
  dBmconv();
  if (dBm != dBmold){
    ucg.setFont(ucg_font_fub17_tr);
    ucg.setColor(0,0,0);
    ucg.setPrintPos(120,180);
    ucg.print(dBmold,1);     
    ucg.setColor(0,240,240);
     ucg.setPrintPos(120,180);   
    ucg.print(dBm,1);
    ucg.setPrintPos(180,180);
    ucg.setColor(255,255,255);
    ucg.print("dBm");
    dBmold=dBm;
 
    }
     counter=0;  
  }
   }
//  DDS_freqset(0);
  EEPROM.put(0x070,SGfreq);
  dBmold=100;
  menu1();
  menu2();
}

// ------------ Frequency Step chenge --------------------------
void step(){
  ucg.setColor(0,0,0);
  ucg.drawBox(220,205,85,30);
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setColor(100,100,100);
  ucg.setPrintPos(140,227);
  ucg.print("STEP [            ]");
  switch(fstepflg){
    case 1:
      fstep=1;
       ucg.setPrintPos(220,230);
       if (configflg==3){
        ucg.print("   1dB");
       }
       else{
        ucg.print("    1Hz");
      }   
    break;
    case 2:
      fstep=10;
      ucg.setPrintPos(220,230);
      if (configflg==3){
        ucg.print("  10dB");
      }
      else {
        ucg.print("  10Hz");  
      }
    break;      
    case 3:
      if(configflg==3){
        fstepflg=2;
        fstep=10;
      ucg.setPrintPos(220,230);        
        ucg.print("  10dB");
      }
        else{
          fstep=100;
          ucg.setPrintPos(220,230);
          ucg.print(" 100Hz");  
        }
    break;
    case 4:
      fstep=1000;
      ucg.setPrintPos(220,230);
      ucg.print("  1kHz");  
    break;
    case 5:
       fstep=10000;
      ucg.setPrintPos(220,230);
      ucg.print(" 10kHz");
      break;
    case 6:
      fstep=100000;
      ucg.setPrintPos(220,230);
      ucg.print("100kHz");
    break;
    case 7:
      fstep=1000000;
      ucg.setPrintPos(220,230);
      ucg.print("  1MHz");
    break;
    case 8:
      fstep=10000000;  
      ucg.setPrintPos(220,230);
      ucg.print(" 10MHz");
    break;
  }
}

// -------------- Power meter -------------------------
void powermeter(){
  mode=2;
  ucg.setColor(255,255,255);
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setPrintPos(10,30);
  ucg.print("Power meter");
  while(digitalRead(sw_menu) ==HIGH){
  ADOUT=analogRead(AD8307);
  dBmconv();
  if (dBm != dBmold){
    ucg.setFont(ucg_font_fub35_tn);
    ucg.setColor(0,0,0);
    if (dBmold <0){
     ucg.setPrintPos(50,100);
    }
    else{
     ucg.setPrintPos(72,100);        
    }
    ucg.print(dBmold,1);
    ucg.setColor(255,100,100);
    if (dBm <0){
     ucg.setPrintPos(50,100);
    }
    else{
     ucg.setPrintPos(72,100);        
    }
    ucg.print(dBm,1);
    ucg.setPrintPos(200,100);
    ucg.setColor(255,255,255);
    ucg.setFont(ucg_font_fub17_tr);  
    ucg.print("dBm");
    float watt=pow(10,dBm/10);
    ucg.setColor(0,0,0);
    ucg.drawBox(150,120,100,120);
    ucg.setColor(255,255,255);
    ucg.setPrintPos(150,150);
    ucg.print(watt,5);
    ucg.setPrintPos(250,150);
    ucg.print("mW");
    float volt = sqrt(watt*50/1000);
    ucg.setPrintPos(150,180);
    ucg.print(volt);
    ucg.setPrintPos(250,180);
    ucg.print("Vrms");
    float dbuv=dBm+106.99;
    ucg.setPrintPos(150,210);
    ucg.print(dbuv);
    ucg.setPrintPos(250,210);
    ucg.print("dBuV");
  }
   dBmold=dBm;
   delay(1000);
}
  dBmold=100; 
  menu1();
  menu2();
} 


//-------------- Frequency Response Analyzer ---------------------
void FRM(){
// mode=3;
  float maxfprint=0;
  float minfprint=0;
  ADOUTmax=0;
  ADOUTmin=1023;
  fraconfig();
  configflg=1;
  Casoldraw();
  noInterrupts();
  if (DDSfmin==0){
    DDSfpit=(DDSfmax/300);
  }
  else {
    DDSfpit=((DDSfmax-DDSfmin)/300);
  }
  while(digitalRead(sw_menu)==HIGH){    
  ucg.setPrintPos(12,20);
  ucg.setFont(ucg_font_7x14B_tr);
  ucg.setColor(255,0,0);
  minfprint=(float)DDSfmin/1000;
  ucg.print(minfprint,3);
  ucg.setPrintPos(250,20);
  maxfprint=(float)DDSfmax/1000;
  ucg.print(maxfprint,3);
  ucg.setColor(255,255,255);
  ucg.setPrintPos(120,20);
  ucg.print("10dB/  ref=");
  ucg.print(offsetpix);
  xold=10;yold=235;
  ucg.setColor(255,255,255);
  ucg.setPrintPos(15,232);
  float span=(float)(DDSfmax-DDSfmin)/1000;
  ucg.print("SPAN=");
  ucg.print(span,3);
  ucg.print("kHz");
    DDS_freq=DDSfmin+(DDSfpit);   //33.33;  2017.10.31NKD
    DDS_freqset(DDS_freq);        //2017.10.31NKD
    delay(1000);                    //2017.10.31NKD
//    ADOUT=analogRead(AD8307);     //2017.11.01NKD
//    ADOUTmax=ADOUT;               //2017.11.01NKD
//    ADOUTmax=0;                   //2017.11.01NKD 
  for (int i=1; i<=300;i++){
    DDS_freq=DDSfmin+(DDSfpit*i);   //33.33;  
    DDS_freqset(DDS_freq);
    delay(1);
    ADOUT=analogRead(AD8307);
    dBmconv();
    int y=26-(dBm*3)+(offsetpix*3);
    y=constrain(y,25,235);
    ucg.setColor(0,255,0);
    ucg.drawLine(xold,yold,(xold+1),y);
    xold=xold+1;
    yold=y;
    if (ADOUT>ADOUTmax){
      ADOUTmax=ADOUT;
      ADOUTmaxf=DDS_freq;
      ADmaxi=i;
    }
    if (ADOUT<ADOUTmin){
        ADOUTmin=ADOUT;
        ADOUTminf=DDS_freq;
        ADmini=i;
    }
  }
  if (fcountflg==0){
    FRMf();    
  }

    if (digitalRead(sw_right)==LOW){
      ucg.setColor(0,0,0);
      ucg.drawBox(10,25,310,240);
      Casoldraw();
      fcountflg=0;
      while(digitalRead(sw_right) == LOW){}
    }
  }
  interrupts();
//  DDS_freqset(0);
  menu1();
  menu2();
  fcountflg=0;
}

//-----------Frequency response sub1 -----------------
 void FRMf(){
  float admaxf=0;
  float adminf=0;

  if (mode==3){
     ucg.setColor(0,255,255);
     ucg.drawVLine((10+ADmaxi),25,210);
     ucg.setColor(250,120,30);
     ucg.drawVLine((10+ADmini),25,210);         
    if((DDSfmax-DDSfmin) <=300000){
      int ADmaxj=0;
      int ADminj=0;
      ucg.setColor (255,255,255);
      ucg.setFont(ucg_font_7x14B_tr);
      ucg.setPrintPos(170,37);
      ucg.print("Now Measuring");
      long DDS_freq=ADOUTmaxf-1000;
      if (DDS_freq <= DDSfmin){DDS_freq=DDSfmin;}     
      for (long j=0;j<=2000;j++){
        DDS_freq=DDS_freq+j;
        if(DDS_freq > DDSfmax){DDS_freq=DDSfmax;}  
        DDS_freqset(DDS_freq);
        ADOUT=analogRead(AD8307);
        if (ADOUT>ADOUTmax){
          ADOUTmax=ADOUT;
          ADOUTmaxf=DDS_freq;
        }
      }
      DDS_freq=ADOUTminf-1000;
      if (DDS_freq < DDSfmin){DDS_freq=DDSfmin;}
      for (long j=0;j<=2000;j++){
        DDS_freq=DDS_freq+j;
       if(DDS_freq > DDSfmax){DDS_freq=DDSfmax;}           
       DDS_freqset(DDS_freq);
       ADOUT=analogRead(AD8307);
        if (ADOUT<ADOUTmin){
          ADOUTmin=ADOUT;
          ADOUTminf=DDS_freq;
        }
      }
      ucg.setColor (0,0,0);
      ucg.setFont(ucg_font_7x14B_tr);
      ucg.setPrintPos(170,37);
      ucg.print("Now Measuring");  
      ucg.setColor (250,120,30);
      ucg.setFont(ucg_font_7x14B_tr);
      ucg.setPrintPos(155,37);
      ADOUT=ADOUTmax;
      dBmconv();
      ucg.setColor(0,255,255);
      ucg.print(dBm);
      ucg.print("dBm ");
      admaxf=(float)ADOUTmaxf/1000;
      ucg.print(admaxf,3);
      ucg.print("kHz");
      ucg.setColor(250,120,30);
      ucg.setPrintPos(155,52);
      ADOUT=ADOUTmin;
      dBmconv();
      ucg.print(dBm);
      ucg.print("dBm ");
      adminf=(float)ADOUTminf/1000;
      ucg.print(adminf,3);
      ucg.print("kHz");
      ucg.setColor(255,255,255);
      ucg.setPrintPos(155,67);
      ucg.print("Delta f=");
      long deltf=fabs(ADOUTminf-ADOUTmaxf);
      ucg.print(deltf);
      ucg.print("Hz");
      fcountflg=1;
      ADOUTmin=1023;
      ADOUTmax=0; 
    }
  }
    if (mode == 4){
      ucg.setColor(0,255,255);              //2017.11.01NKD
      ucg.drawVLine((10+ADmaxi),25,210);    //2017.11.01NKD
      ucg.setColor(250,120,30);
      ucg.drawVLine((10+ADmini),25,210);
      float dBmax,dBmin;
      ucg.setFont(ucg_font_7x14B_tr);
      ucg.setPrintPos(155,37);
      ucg.setColor(0,255,255);
      ADOUT=ADOUTmax;
      dBmconv();
      ucg.print(dBm);
      dBmax=dBm;      
      ucg.print("dBm  ");
      admaxf=(float)ADOUTmaxf/1000;
      ucg.print(admaxf,3);
      ucg.print("kHz");
      ucg.setPrintPos(155,52);
       ucg.setColor(250,120,30);
      ADOUT=ADOUTmin;
      dBmconv();
      ucg.print(dBm);
      dBmin=dBm;      
      ucg.print("dBm  ");
      adminf=(float)ADOUTminf/1000;
      ucg.print(adminf,3);
      ucg.print("kHz");
      float rho=0;
      rho=dBmax-dBmin;
      rho=rho/20;
      rho=pow(10,rho);
      swr=-1*((1+rho)/(1-rho));
      ucg.setColor(255,255,255);
      ucg.setPrintPos(155,67);
      ucg.print("SWR = ");
      ucg.print(swr);
      fcountflg=1;
      ADOUTmin=1023;
      ADOUTmax=0;    
  }
 }
//-----------Frequency Response sub2 -----------------------
void fraconfig(){

  noInterrupts();
  float maxfprint=0;
  float minfprint=0;
  ucg.clearScreen();
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setPrintPos(20,25);
  configflg=1;
  if (mode==3){
    ucg.print("Frequency Analyzer Config");
    EEPROM.get(0x040,DDSfmin);
    EEPROM.get(0x050,DDSfmax);
    EEPROM.get(0x060,offsetpix);
    fstepflg=EEPROM.read(0x85);
   }
  if (mode==4){
    ucg.print("Antenna Analyzer Config");
    EEPROM.get(0x045,DDSfmin);
    EEPROM.get(0x055,DDSfmax);
    EEPROM.get(0x065,offsetpix);
    fstepflg=EEPROM.read(0x90);
  }
  ucg.setColor(255,255,255);
  ucg.setPrintPos(10,75);
  ucg.print("1.Start ");
  ucg.setPrintPos(150,75);
  minfprint=(float)DDSfmin/1000;
  ucg.print(minfprint,3);
  ucg.print("kHz"); 
  ucg.setPrintPos(10,100);
  ucg.print("2.Stop ");
  ucg.setPrintPos(150,100);
  maxfprint=(float)DDSfmax/1000;
  ucg.print(maxfprint,3);
  ucg.print("kHz");   
  ucg.setPrintPos(10,125);
  ucg.print("3.Ref set");
  ucg.setPrintPos(150,125);
  ucg.print(offsetpix);
  while (digitalRead(sw_menu) == HIGH){ 
    if (digitalRead(sw_left) == LOW){
       configflg = configflg-1;
      while(digitalRead(sw_left) == LOW){}
    }
    if (digitalRead(sw_right) == LOW){
       configflg=configflg+1;
      while(digitalRead(sw_right) == LOW){}
    }
    configflg=constrain(configflg,1,3);
    switch(configflg){
    case 1:
      ucg.setPrintPos(10,160);
      ucg.setColor(255,255,255);
      ucg.print("[");
      ucg.setColor(255,0,0);
      ucg.print(" 1");
      ucg.setColor(255,255,255);
      ucg.print(" 2 3 ]");
      break;
    case 2:
      ucg.setPrintPos(10,160);
      ucg.setColor(255,255,255);
      ucg.print("[ 1");
      ucg.setColor(255,0,0);
      ucg.print(" 2");
      ucg.setColor (255,255,255);
      ucg.print(" 3 ]");  
    break;
    case 3:
      ucg.setPrintPos(10,160);
      ucg.setColor(255,255,255);
      ucg.print("[ 1 2");
      ucg.setColor(255,0,0);
      ucg.print(" 3");
      ucg.setColor(255,255,255);
      ucg.print(" ]");  
    break;
    }    
    if (digitalRead(sw_enter) ==LOW){
      fraconfigin();
    }
  }
  ucg.clearScreen();
  interrupts();
}

//-------------- Frequency Response sub3 -------------------
void fraconfigin(){
  float minfprint=0;
  float maxfprint=0;
  interrupts();
  ucg.setColor(0,240,240);
  ucg.drawFrame(160,170,150,35);  
    switch (configflg){
      case 1:
        ucg.setPrintPos(140,160);
        ucg.print("Start ");
        ucg.setPrintPos(180,195);
        minfprint=(float)DDSfmin/1000;
        ucg.print(minfprint,3);
        step();
      break;
      case 2:
        ucg.setPrintPos(140,160);
        ucg.print("Stop ");
        ucg.setPrintPos(180,195);
        maxfprint=(float)DDSfmax/1000;
        ucg.print(maxfprint,3);
        step();
      break;
      case 3:
        ucg.setPrintPos(140,160);
        ucg.print("Ref set ");
        ucg.setPrintPos(180,195);
        ucg.print(offsetpix);
        fstepflg=1;
        step();
        break;   
    }
    while(digitalRead(sw_enter)==HIGH){
      if (digitalRead(sw_left) == LOW){
        fstepflg=fstepflg-1;
        if (fstepflg <1){fstepflg=1;}
          while(digitalRead(sw_left)==LOW){}
          step();
          ucg.setColor(0,240,240);       
        }
      if (digitalRead(sw_right) == LOW){
        fstepflg=fstepflg+1;
        if (fstepflg >8){fstepflg=8;}
          while(digitalRead(sw_right)==LOW){}
          step();
          ucg.setColor(0,240,240);         
       }
      if (configflg != 3){
        if(mode==3){
          EEPROM.write(0x085,fstepflg);
        }
        if(mode==4){
          EEPROM.write(0x090,fstepflg);
        }
      }
    }
       if(mode==3){
        fstepflg=EEPROM.read(0x085);
       }
       if(mode==4){
        fstepflg=EEPROM.read(0x090);
       }
  if(DDSfmax < DDSfmin){
    DDSfmax=DDSfmin; 
  }
  if (mode==3){
    EEPROM.put(0x040,DDSfmin);
    EEPROM.put(0x050,DDSfmax);
    EEPROM.put(0x060,offsetpix);
  } 
  if (mode==4){
    EEPROM.put(0x045,DDSfmin);
    EEPROM.put(0x055,DDSfmax);
    EEPROM.put(0x065,offsetpix);
  }
  ucg.setColor(0,0,0);
  ucg.drawBox(135,140,185,100);
  ucg.drawBox(150,50,170,100);
  ucg.setColor(255,255,255);
  ucg.setPrintPos(150,75);
  minfprint=(float)DDSfmin/1000;
  ucg.print(minfprint,3);
  ucg.print("kHz");
  ucg.setPrintPos(150,100);
  maxfprint=(float)DDSfmax/1000;
  ucg.print(maxfprint,3);
  ucg.print("kHz");
  ucg.setPrintPos(150,125);
  ucg.print(offsetpix);
  ucg.print("dBm"); 
  noInterrupts();        
}

// ---------------- DDS Initialize ---------------------

void DDS_init(){
  digitalWrite(DDS_RST,HIGH);
  delay(1);
  digitalWrite(DDS_RST,LOW);
  digitalWrite(DDS_W_CLK,HIGH);
  digitalWrite(DDS_W_CLK,LOW);
  digitalWrite(DDS_FQ_UD,HIGH); 
  digitalWrite(DDS_FQ_UD,LOW);
}

// ---------------- DDS Frequency set -----------------

void DDS_freqset(double frequency){
  unsigned long wrk =frequency * TWO_E / DDS_CLK;
  Serial.println(wrk); 
  digitalWrite(DDS_FQ_UD,HIGH);
  digitalWrite(DDS_FQ_UD,LOW);
  shiftOut(DDS_DATA , DDS_W_CLK,LSBFIRST,wrk);
  shiftOut(DDS_DATA , DDS_W_CLK,LSBFIRST,(wrk >> 8));
  shiftOut(DDS_DATA , DDS_W_CLK,LSBFIRST,(wrk >> 16));
  shiftOut(DDS_DATA , DDS_W_CLK,LSBFIRST,(wrk >> 24));
  shiftOut(DDS_DATA , DDS_W_CLK,LSBFIRST,DDS_CMD);
  digitalWrite(DDS_FQ_UD,HIGH);
  digitalWrite(DDS_FQ_UD,LOW);
}
//------------- FRA Frame draw -----------------------------
void Casoldraw(){
  ucg.setColor(100,100,100);
  for (int i=0; i<=10; i++){
    ucg.drawVLine(10+(i*30),25,210);
  }
  for (int i=0; i<=7;i++){
     ucg.drawHLine(10,(235-(i*30)),300);
  }
}

//-------------- dB Converter -------------------------------
void dBmconv(){
  dBm=(ADOUT*AD_a)+AD_b; //    dBm=(ADOUT*0.09)-80.91;
}

//--------------- AD8307 Ajustment --------------------------
void ADcheck(){
  mode=6;
  ucg.setColor(255,255,255);
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setPrintPos(10,30);
  ucg.print("AD8307 Adjustment");
  float x1,x2; 
  ucg.setFont(ucg_font_7x14B_tr);
  ucg.setPrintPos(20,60);
  ucg.print("OLD AD_a =");
  ucg.print(AD_a);
  ucg.print("  OLD Ad_b =");
  ucg.print(AD_b);
  ucg.setPrintPos(20,80);
  ucg.print("Input 0dBm");
  ucg.setPrintPos(20,100);  
  ucg.print("10seconds later Push [ENTER]");
  while(digitalRead(sw_enter)==HIGH){}
  x1=analogRead(AD8307);
  ucg.setPrintPos(20,120);
  ucg.print("AD8307Read X1=");
  ucg.print(x1);
  ucg.setPrintPos(20,140);
  ucg.print("Input -50dBm");
  ucg.setPrintPos(20,160);
  ucg.print("10seconds after Push {[ENTER]");
  while(digitalRead(sw_enter)==HIGH){}
  x2=analogRead(AD8307);  
  ucg.setPrintPos(20,180);    
  ucg.print("AD8307Read X2=");
  ucg.print(x2);
  AD_a=50/(x1-x2);
  AD_b=0-(AD_a*x1);
  ucg.setPrintPos(20,200);
  ucg.print("AD_a =");
  ucg.print(AD_a);
  ucg.print("  Ad_b =");
  ucg.print(AD_b);
  EEPROM.put(0x100,AD_a);
  EEPROM.put(0x120,AD_b);
  ucg.setPrintPos(20,220);
  ucg.print("Push [MENU] back to TOP");
  while(digitalRead(sw_menu)==HIGH){}
  menu1();
  menu2();
}

// -------------- DDS CLOCK Ajustment ---------------
void DDSajust(){
  mode=5;
  fstep=3;
  EEPROM.get(0x010,DDS_CLK);
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setColor(0,240,240);
  ucg.setPrintPos(180,195);
  ucg.print(DDS_CLK);
  DDS_freqset(10000000);    
  ucg.setColor(255,255,255);
  ucg.setFont(ucg_font_fub17_tr);
  ucg.setPrintPos(10,30);
  ucg.print("DDS CLOCK Adjustment");
  ucg.setFont(ucg_font_7x14B_tr);  
  ucg.setPrintPos(10,70);
  ucg.print("OUTPUT Frequency of DDS is 10.000000MHz");
  ucg.setPrintPos(10,85);  
  ucg.print("Connect to frequency Counter");
  ucg.setPrintPos(10,100);  
  ucg.print("Adjust a frequency of DDS");
  ucg.setPrintPos(10,115);  
  ucg.print("      as a counter will be 10.000000MHz");
  ucg.setColor(0,240,240);
  ucg.drawFrame(160,170,150,35);
  fstepflg=3;
  step();  
  while(digitalRead(sw_menu)==HIGH){
    if (digitalRead(sw_left) == LOW){
      fstepflg=fstepflg-1;
      if (fstepflg <1){fstepflg=1;}
        step();
          ucg.setColor(0,240,240);       
    }
    while(digitalRead(sw_left)==LOW){}
    if (digitalRead(sw_right) == LOW){
      fstepflg=fstepflg+1;
      if (fstepflg >8){fstepflg=8;}
      step();
      ucg.setColor(0,240,240);         
    }
    while(digitalRead(sw_right)==LOW){}
  }
  while(digitalRead(sw_menu)==LOW){};
  menu1();
  menu2();
} 

void eepinitwrite(){
  DDS_CLK=180000000;
  AD_a=0.09;
  AD_b=-86.06;
  DDSfmin=0;
  DDSfmax=55000000;
  offsetpix=0;
  SGfreq=10000000;
    EEPROM.put(0x010,DDS_CLK);
    EEPROM.put(0x100,AD_a);
    EEPROM.put(0x120,AD_b);
    EEPROM.put(0x040,DDSfmin);
    EEPROM.put(0x050,DDSfmax);
    EEPROM.put(0x060,offsetpix);
    EEPROM.put(0x045,DDSfmin);
    EEPROM.put(0x055,DDSfmax);
    EEPROM.put(0x065,offsetpix);
    EEPROM.put(0x070,SGfreq);
    EEPROM.write(0x080,7);
    EEPROM.write(0x085,7);
    EEPROM.write(0x090,7);
}

